/*
 * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.theme;

import org.jvnet.substance.color.ColorScheme;

/**
 * Base class for themes that are based on some other theme. This class is part
 * of officially supported API.
 * 
 * @author Kirill Grouchnikov
 */
public abstract class SubstanceWrapperTheme extends SubstanceTheme {
	/**
	 * The original (wrapped) theme.
	 */
	protected SubstanceTheme originalTheme;

	/**
	 * Theme for controls in default visual state.
	 */
	protected SubstanceTheme defaultTheme;

	/**
	 * Theme for controls in disabled visual state.
	 */
	protected SubstanceTheme disabledTheme;

	/**
	 * Theme for borders.
	 */
	protected SubstanceTheme borderTheme;

	/**
	 * Creates a new wrapper theme.
	 * 
	 * @param originalTheme
	 *            The original theme.
	 * @param colorScheme
	 *            The associated color scheme.
	 * @param displayName
	 *            Display name for the wrapper theme.
	 * @param themeKind
	 *            Theme kind for the wrapper theme.
	 */
	public SubstanceWrapperTheme(SubstanceTheme originalTheme,
			ColorScheme colorScheme, String displayName, ThemeKind themeKind) {
		super(colorScheme, displayName, themeKind);
		this.originalTheme = originalTheme;
	}

	/**
	 * Returns the original (wrapped) theme.
	 * 
	 * @return The original (wrapped) theme.
	 */
	public SubstanceTheme getOriginalTheme() {
		return this.originalTheme;
	}
}
