/*
 * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.utils.menu;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.geom.Area;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.plaf.ButtonUI;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;

import org.jvnet.lafwidget.layout.TransitionLayout;
import org.jvnet.substance.*;
import org.jvnet.substance.painter.text.SubstanceTextPainter;
import org.jvnet.substance.utils.SubstanceCoreUtilities;
import org.jvnet.substance.utils.SubstanceConstants.MenuGutterFillKind;

/**
 * A collection of ugly finctions from
 * {@link javax.swing.plaf.basic.BasicMenuItemUI} since they are private. This
 * class is <b>for internal use only</b>.
 * 
 * @author Kirill Grouchnikov
 */
public class MenuUtilities {
	/**
	 * Delegate for painting the background.
	 */
	private static SubstanceMenuBackgroundDelegate backgroundDelegate = new SubstanceMenuBackgroundDelegate(
			0.4f);

	public static final String LAYOUT_METRICS = "substancelaf.internal.menus.layoutMetrics";

	public static final String GUTTER_X = "substancelaf.internal.menus.gutterX";

	public static class MenuPropertyListener implements PropertyChangeListener {
		protected JMenuItem menuItem;

		public MenuPropertyListener(JMenuItem menuItem) {
			this.menuItem = menuItem;
		}

		public void install() {
			this.menuItem.addPropertyChangeListener(this);
		}

		public void uninstall() {
			this.menuItem.removePropertyChangeListener(this);
		}

		public void propertyChange(PropertyChangeEvent evt) {
			// if (!evt.getPropertyName().equals(
			// SubstanceMenu.SINGLE_MENU_TEXT_OFFSET))
			// SwingUtilities.invokeLater(new Runnable() {
			// public void run() {
			// menuItem.putClientProperty(
			// SubstanceMenu.SINGLE_MENU_TEXT_OFFSET, null);
			// if (menuItem.getParent() instanceof JPopupMenu) {
			// JPopupMenu parent = (JPopupMenu) menuItem
			// .getParent();
			// parent
			// .putClientProperty(
			// SubstanceMenu.SINGLE_MENU_TEXT_OFFSET,
			// null);
			// }
			// }
			// });
			if (!evt.getPropertyName().equals(MenuUtilities.LAYOUT_METRICS)) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						MenuUtilities.cleanPopupLayoutMetrics(menuItem);
					}
				});
			}
		}

	}

	public static class MenuLayoutInfo {
		public Rectangle viewRect;
		public Rectangle iconRect;
		public Rectangle checkIconRect;
		public Rectangle textRect;
		public Rectangle acceleratorRect;
		public Rectangle arrowIconRect;
		public String text;
	}

	public static class MenuLayoutMetrics {
		public int maxIconWidth;
		public int maxCheckIconWidth;
		public int maxTextWidth;
		public int maxAcceleratorWidth;
		public int maxArrowIconWidth;
		public int maxGap;
	}

	/* Client Property keys for the icon and text maximal offsets */
	// static final StringBuffer MAX_ICON_OFFSET = new StringBuffer(
	// "maxIconOffset");
	//
	// static final StringBuffer MAX_TEXT_OFFSET = new StringBuffer(
	// "maxTextOffset");
	//
	// public static int getTextOffset(Component menuItemParent) {
	// if (!(menuItemParent instanceof JPopupMenu)) {
	// return 0;
	// }
	//
	// return getTextOffset(null, menuItemParent);
	// }
	// private static int getSingleItemTextOffset(JMenuItem menuItem) {
	// // Check if we have this property on the menu item.
	// Object prop = menuItem
	// .getClientProperty(SubstanceMenu.SINGLE_MENU_TEXT_OFFSET);
	// if (prop instanceof Integer) {
	// // This property is reset on every property change
	// // to the menu item. If the property exists, nothing
	// // has changed since the last time it was computed.
	// return (Integer) prop;
	// }
	//
	// SubstanceMenu ui = (SubstanceMenu) menuItem.getUI();
	// int result = MenuUtilities.getTextOffset(menuItem, ui
	// .getAcceleratorFont(), ui.getCheckIcon(), ui.getArrowIcon(), ui
	// .getDefaultTextIconGap());
	// // Now that the offset has been computed, set it on the menu item.
	// menuItem.putClientProperty(SubstanceMenu.SINGLE_MENU_TEXT_OFFSET,
	// result);
	// return result;
	// }
	// public static int getTextOffset(JMenuItem menuItem) {
	// return getTextOffset(menuItem, menuItem.getParent());
	// }
	//
	// public static int getTextOffset(JMenuItem menuItem, Component parent) {
	// if (!(parent instanceof JPopupMenu)) {
	// return getSingleItemTextOffset(menuItem);
	// }
	//
	// // test if there's a computed property on the parent
	// JPopupMenu popupMenu = (JPopupMenu) parent;
	// Object prop = popupMenu
	// .getClientProperty(SubstanceMenu.SINGLE_MENU_TEXT_OFFSET);
	// if (prop instanceof Integer) {
	// // This property is reset on every change to both the
	// // popup (adding / removing child menu component) and
	// // and each child component (on any property change).
	// // So, if this property exists on the popup menu, nothing
	// // has changed since the last time the offset was computed.
	// return (Integer) prop;
	// }
	// int maxTextOffset = -1;
	// int maxIconWidth = 0;
	// int maxIconX = 0;
	// int maxCheckIconX = 0;
	// int maxCheckIconWidth = 0;
	// // int maxViewRect = 0;
	// for (int i = 0; i < popupMenu.getComponentCount(); i++) {
	// Component comp = popupMenu.getComponent(i);
	// if (comp instanceof JMenuItem) {
	// JMenuItem childMenuItem = (JMenuItem) comp;
	// ButtonUI bui = childMenuItem.getUI();
	// if (!(bui instanceof SubstanceMenu))
	// continue;
	// SubstanceMenu ui = (SubstanceMenu) bui;
	// MenuLayoutInfo mli = MenuUtilities.getMenuLayoutInfo(
	// childMenuItem, ui.getAcceleratorFont(), ui
	// .getCheckIcon(), ui.getArrowIcon(), ui
	// .getDefaultTextIconGap());
	//
	// if (popupMenu.getComponentOrientation().isLeftToRight()) {
	// if (mli.textRect != null) {
	// maxTextOffset = Math.max(maxTextOffset, mli.textRect.x);
	// }
	// if (mli.iconRect != null) {
	// maxIconWidth = Math
	// .max(maxIconWidth, mli.iconRect.width
	// + ui.getDefaultTextIconGap());
	// maxIconX = Math.max(maxIconX, mli.iconRect.x);
	// }
	// if (mli.checkIconRect != null) {
	// maxCheckIconWidth = Math.max(maxCheckIconWidth,
	// mli.checkIconRect.width + 2
	// * ui.getDefaultTextIconGap());
	// maxCheckIconX = Math.max(maxCheckIconX,
	// mli.checkIconRect.x);
	// }
	// } else {
	// if (mli.textRect != null) {
	// maxTextOffset = Math.max(maxTextOffset, mli.viewRect.x
	// + mli.viewRect.width
	// - (mli.textRect.x + mli.textRect.width));
	// }
	// if (mli.iconRect != null) {
	// maxIconWidth = Math.max(maxIconWidth,
	// mli.iconRect.width + 2
	// * ui.getDefaultTextIconGap());
	// maxIconX = Math.max(maxIconX, mli.iconRect.x
	// + mli.iconRect.width);
	// }
	// if (mli.checkIconRect != null) {
	// maxCheckIconWidth = Math.max(maxCheckIconWidth,
	// mli.checkIconRect.width
	// + ui.getDefaultTextIconGap());
	// maxCheckIconX = Math.max(maxCheckIconX,
	// mli.checkIconRect.x + mli.checkIconRect.width);
	// }
	// // maxViewRect = Math.max(maxViewRect, mli.viewRect.x
	// // + mli.viewRect.width);
	// }
	//
	// //
	// // int childResult = MenuUtilities.getTextOffset(g,
	// // childMenuItem,
	// // ui.getAcceleratorFont(), ui.getCheckIcon(), ui
	// // .getArrowIcon(), ui.getDefaultTextIconGap());
	// // if (maxOffset < childResult)
	// // maxOffset = childResult;
	// }
	// }
	// // Now that the offset is computed, store it on the
	// // popup menu.
	// int maxOffset = Math.max(maxTextOffset, maxIconWidth
	// + maxCheckIconWidth);
	// if (popupMenu.getComponentOrientation().isLeftToRight()) {
	// maxOffset = Math.max(maxOffset, maxIconX + maxIconWidth);
	// maxOffset = Math.max(maxOffset, maxCheckIconX + maxCheckIconWidth);
	// } else {
	// // maxOffset = Math.max(maxOffset, maxViewRect
	// // - (maxIconX - maxIconWidth));
	// }
	// popupMenu.putClientProperty(SubstanceMenu.SINGLE_MENU_TEXT_OFFSET,
	// maxOffset);
	// return maxOffset;
	// }
	//
	// public static int getTextOffset(JMenuItem menuItem, Font acceleratorFont,
	// Icon checkIcon, Icon arrowIcon, int defaultTextIconGap) {
	//
	// // Dimension size = b.getSize();
	// // int menuWidth = menuItem.getWidth();
	// // int menuHeight = menuItem.getHeight();
	// Insets i = menuItem.getInsets();
	//
	// Rectangle iconRect = new Rectangle();
	// Rectangle textRect = new Rectangle();
	// Rectangle acceleratorRect = new Rectangle();
	// Rectangle checkIconRect = new Rectangle();
	// Rectangle arrowIconRect = new Rectangle();
	// Rectangle viewRect = new Rectangle(Short.MAX_VALUE, Short.MAX_VALUE);
	//
	// int menuWidth = menuItem.getWidth();
	// int menuHeight = menuItem.getHeight();
	// if ((menuWidth > 0) && (menuHeight > 0))
	// viewRect.setBounds(0, 0, menuWidth, menuHeight);
	//
	// // fix for defect 108 - failed on b62 of Mustang and later on Hebrew
	// // localized menu strings.
	// // viewRect.setBounds(0, 0, menuWidth, menuHeight);
	//
	// viewRect.x += i.left;
	// viewRect.y += i.top;
	// viewRect.width -= (i.right + viewRect.x);
	// viewRect.height -= (i.bottom + viewRect.y);
	//
	// FontMetrics fm = menuItem.getFontMetrics(menuItem.getFont());
	// FontMetrics fmAccel = menuItem.getFontMetrics(acceleratorFont);
	//
	// // get Accelerator text
	// KeyStroke accelerator = menuItem.getAccelerator();
	// String acceleratorText = "";
	// if (accelerator != null) {
	// int modifiers = accelerator.getModifiers();
	// if (modifiers > 0) {
	// acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
	// // acceleratorText += "-";
	// acceleratorText += UIManager
	// .getString("MenuItem.acceleratorDelimiter");
	// }
	//
	// int keyCode = accelerator.getKeyCode();
	// if (keyCode != 0) {
	// acceleratorText += KeyEvent.getKeyText(keyCode);
	// } else {
	// acceleratorText += accelerator.getKeyChar();
	// }
	// }
	//
	// // layout the text and icon
	// layoutMenuItem(menuItem, fm, menuItem.getText(), fmAccel,
	// acceleratorText, menuItem.getIcon(), checkIcon, arrowIcon,
	// menuItem.getVerticalAlignment(), menuItem
	// .getHorizontalAlignment(), menuItem
	// .getVerticalTextPosition(), menuItem
	// .getHorizontalTextPosition(), viewRect, iconRect,
	// textRect, acceleratorRect, checkIconRect, arrowIconRect,
	// menuItem.getText() == null ? 0 : defaultTextIconGap,
	// defaultTextIconGap);
	//
	// if (menuItem.getComponentOrientation().isLeftToRight()) {
	// return textRect.x;
	// } else {
	// // int result = viewRect.x + viewRect.width;
	// // if (iconRect.width != 0) {
	// // result -= (iconRect.x + defaultTextIconGap);
	// // }
	// // else {
	// // if (checkIconRect.width != 0) {
	// // result -= (checkIconRect.x + defaultTextIconGap);
	// // }
	// // }
	// return viewRect.x + viewRect.width - iconRect.x
	// - ((iconRect.width != 0) ? 0 : defaultTextIconGap);
	// }
	// }
	public static MenuLayoutInfo getMenuLayoutInfo(
			// Graphics g,
			JMenuItem menuItem, Font acceleratorFont, Icon checkIcon,
			Icon arrowIcon, int defaultTextIconGap) {

		// Dimension size = b.getSize();
		// int menuWidth = menuItem.getWidth();
		// int menuHeight = menuItem.getHeight();
		Insets i = menuItem.getInsets();

		Rectangle iconRect = new Rectangle(0, 0, 0, 0);
		Rectangle textRect = new Rectangle(0, 0, 0, 0);
		Rectangle acceleratorRect = new Rectangle(0, 0, 0, 0);
		Rectangle checkIconRect = new Rectangle(0, 0, 0, 0);
		Rectangle arrowIconRect = new Rectangle(0, 0, 0, 0);
		Rectangle viewRect = new Rectangle(Short.MAX_VALUE, Short.MAX_VALUE);

		int menuWidth = menuItem.getWidth();
		int menuHeight = menuItem.getHeight();
		if ((menuWidth > 0) && (menuHeight > 0))
			viewRect.setBounds(0, 0, menuWidth, menuHeight);

		// fix for defect 108 - failed on b62 of Mustang and later on Hebrew
		// localized menu strings.
		// viewRect.setBounds(0, 0, menuWidth, menuHeight);

		viewRect.x += i.left;
		viewRect.y += i.top;
		viewRect.width -= (i.right + viewRect.x);
		viewRect.height -= (i.bottom + viewRect.y);

		// Font f = menuItem.getFont();
		// g.setFont(f);
		FontMetrics fm = menuItem.getFontMetrics(menuItem.getFont());
		FontMetrics fmAccel = menuItem.getFontMetrics(acceleratorFont);

		// get Accelerator text
		KeyStroke accelerator = menuItem.getAccelerator();
		String acceleratorText = "";
		if (accelerator != null) {
			int modifiers = accelerator.getModifiers();
			if (modifiers > 0) {
				acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
				// acceleratorText += "-";
				acceleratorText += UIManager
						.getString("MenuItem.acceleratorDelimiter");
			}

			int keyCode = accelerator.getKeyCode();
			if (keyCode != 0) {
				acceleratorText += KeyEvent.getKeyText(keyCode);
			} else {
				acceleratorText += accelerator.getKeyChar();
			}
		}

		// layout the text and icon
		String text = layoutMenuItem(menuItem, fm, menuItem.getText(), fmAccel,
				acceleratorText, menuItem.getIcon(), checkIcon, arrowIcon,
				menuItem.getVerticalAlignment(), menuItem
						.getHorizontalAlignment(), menuItem
						.getVerticalTextPosition(), menuItem
						.getHorizontalTextPosition(), viewRect, iconRect,
				textRect, acceleratorRect, checkIconRect, arrowIconRect,
				menuItem.getText() == null ? 0 : defaultTextIconGap,
				defaultTextIconGap);

		MenuLayoutInfo mlInfo = new MenuLayoutInfo();
		// mlInfo.checkIconRect = (checkIconRect.width > 0) ? checkIconRect :
		// null;
		// mlInfo.iconRect = (iconRect.width > 0) ? iconRect : null;
		// mlInfo.textRect = (textRect.width > 0) ? textRect : null;
		// mlInfo.viewRect = (viewRect.width > 0) ? viewRect : null;
		mlInfo.checkIconRect = checkIconRect;
		mlInfo.iconRect = iconRect;
		mlInfo.textRect = textRect;
		mlInfo.viewRect = viewRect;
		mlInfo.acceleratorRect = acceleratorRect;
		mlInfo.arrowIconRect = arrowIconRect;
		mlInfo.text = text;

		return mlInfo;
		//		
		// if (menuItem.getComponentOrientation().isLeftToRight()) {
		// return textRect.x;
		// } else {
		// // int result = viewRect.x + viewRect.width;
		// // if (iconRect.width != 0) {
		// // result -= (iconRect.x + defaultTextIconGap);
		// // }
		// // else {
		// // if (checkIconRect.width != 0) {
		// // result -= (checkIconRect.x + defaultTextIconGap);
		// // }
		// // }
		// return viewRect.x + viewRect.width - iconRect.x
		// - ((iconRect.width != 0) ? 0 : defaultTextIconGap);
		// }
	}

	/**
	 * Compute and return the location of the icons origin, the location of
	 * origin of the text baseline, and a possibly clipped version of the
	 * compound labels string. Locations are computed relative to the viewRect
	 * rectangle.
	 */
	private static String layoutMenuItem(JMenuItem menuItem, FontMetrics fm,
			String text, FontMetrics fmAccel, String acceleratorText,
			Icon icon, Icon checkIcon, Icon arrowIcon, int verticalAlignment,
			int horizontalAlignment, int verticalTextPosition,
			int horizontalTextPosition, Rectangle viewRect, Rectangle iconRect,
			Rectangle textRect, Rectangle acceleratorRect,
			Rectangle checkIconRect, Rectangle arrowIconRect, int textIconGap,
			int menuItemGap) {

		SwingUtilities.layoutCompoundLabel(menuItem, fm, text, icon,
				verticalAlignment, horizontalAlignment, verticalTextPosition,
				horizontalTextPosition, viewRect, iconRect, textRect,
				textIconGap);

		/*
		 * Initialize the acceelratorText bounds rectangle textRect. If a null
		 * or and empty String was specified we substitute "" here and use
		 * 0,0,0,0 for acceleratorTextRect.
		 */
		if ((acceleratorText == null) || acceleratorText.equals("")) {
			acceleratorRect.width = acceleratorRect.height = 0;
			acceleratorText = "";
		} else {
			acceleratorRect.width = fmAccel.stringWidth(acceleratorText);
			acceleratorRect.height = fmAccel.getHeight();
		}

		/*
		 * Initialize the checkIcon bounds rectangle's width & height.
		 */

		if (useCheckAndArrow(menuItem)) {
			if (checkIcon != null) {
				checkIconRect.width = checkIcon.getIconWidth();
				checkIconRect.height = checkIcon.getIconHeight();
			} else {
				checkIconRect.width = checkIconRect.height = 0;
			}

			/*
			 * Initialize the arrowIcon bounds rectangle width & height.
			 */

			if (arrowIcon != null) {
				arrowIconRect.width = arrowIcon.getIconWidth();
				arrowIconRect.height = arrowIcon.getIconHeight();
			} else {
				arrowIconRect.width = arrowIconRect.height = 0;
			}
		}

		Rectangle labelRect = iconRect.union(textRect);
		if (menuItem.getComponentOrientation().isLeftToRight()) {
			textRect.x += menuItemGap;
			iconRect.x += menuItemGap;

			// Position the Accelerator text rect
			acceleratorRect.x = viewRect.x + viewRect.width
					- arrowIconRect.width - menuItemGap - acceleratorRect.width;

			// Position the Check and Arrow Icons
			if (useCheckAndArrow(menuItem)) {
				checkIconRect.x = viewRect.x + menuItemGap;
				textRect.x += menuItemGap + checkIconRect.width;
				iconRect.x += menuItemGap + checkIconRect.width;
				arrowIconRect.x = viewRect.x + viewRect.width - menuItemGap
						- arrowIconRect.width;
			}
		} else {
			textRect.x -= menuItemGap;
			iconRect.x -= menuItemGap;

			// Position the Accelerator text rect
			acceleratorRect.x = viewRect.x + arrowIconRect.width + menuItemGap;

			// Position the Check and Arrow Icons
			if (useCheckAndArrow(menuItem)) {
				checkIconRect.x = viewRect.x + viewRect.width - menuItemGap
						- checkIconRect.width;
				textRect.x -= menuItemGap + checkIconRect.width;
				iconRect.x -= menuItemGap + checkIconRect.width;
				arrowIconRect.x = viewRect.x + menuItemGap;
			}
		}

		// Align the accelertor text and the check and arrow icons vertically
		// with the center of the label rect.
		acceleratorRect.y = labelRect.y + (labelRect.height / 2)
				- (acceleratorRect.height / 2);
		if (useCheckAndArrow(menuItem)) {
			arrowIconRect.y = labelRect.y + (labelRect.height / 2)
					- (arrowIconRect.height / 2);
			checkIconRect.y = labelRect.y + (labelRect.height / 2)
					- (checkIconRect.height / 2);
		}

		/*
		 * System.out.println("Layout: text="+menuItem.getText()+"\n\tv="
		 * +viewRect+"\n\tc="+checkIconRect+"\n\ti="
		 * +iconRect+"\n\tt="+textRect+"\n\tacc="
		 * +acceleratorRect+"\n\ta="+arrowIconRect+"\n");
		 */

		return text;
	}

	/*
	 * Returns false if the component is a JMenu and it is a top level menu (on
	 * the menubar).
	 */
	private static boolean useCheckAndArrow(JMenuItem menuItem) {
		boolean b = true;
		if ((menuItem instanceof JMenu)
				&& (((JMenu) menuItem).isTopLevelMenu())) {
			b = false;
		}
		return b;
	}

	private static JComponent getMenuItemParent(JMenuItem mi) {
		Container parent = mi.getParent();
		if ((parent instanceof JComponent)
				&& (!(mi instanceof JMenu) || !((JMenu) mi).isTopLevelMenu())) {
			return (JComponent) parent;
		} else {
			return null;
		}
	}

	// public static void lineupTextRectangle(JMenuItem menuItem,
	// Rectangle textRect, Font acceleratorFont, Icon checkIcon,
	// Icon arrowIcon, int defaultTextIconGap) {
	// if (textRect == null)
	// return;
	// int textOffset = MenuUtilities.getTextOffset(menuItem);
	// // System.out.println()
	// if (menuItem.getComponentOrientation().isLeftToRight()) {
	// textRect.setLocation(textOffset, textRect.y);
	// } else {
	// int loc = menuItem.getWidth() - textOffset - textRect.width
	// - defaultTextIconGap;
	// textRect.setLocation(loc, textRect.y);
	// }
	//
	// }

	public static void paintMenuItem(Graphics g, JMenuItem menuItem,
			Icon checkIcon, Icon arrowIcon, int defaultTextIconGap) {
		Graphics2D g2d = (Graphics2D) g.create();

		// special handling of menu items covered by lightweight popups
		// for the defect 297
		if (Boolean.TRUE
				.equals(menuItem
						.getClientProperty(SubstanceCoreUtilities.IS_COVERED_BY_LIGHTWEIGHT_POPUPS))) {
			JRootPane rootPane = SwingUtilities.getRootPane(menuItem);
			if (rootPane != null) {
				Component[] popups = rootPane.getLayeredPane()
						.getComponentsInLayer(JLayeredPane.POPUP_LAYER);
				if (popups != null) {
					int popupIndexToStartWith = SubstanceCoreUtilities
							.getPopupParentIndexOf(menuItem, popups) - 1;
					Area clip = new Area(g2d.getClip());
					for (int i = popupIndexToStartWith; i >= 0; i--) {
						Component popup = popups[i];
						// convert the popup bounds to the menu item coordinate
						// system
						Rectangle popupArea = SwingUtilities.convertRectangle(
								rootPane.getLayeredPane(), popup.getBounds(),
								menuItem);
						// and subtract this area from the clip
						clip.subtract(new Area(popupArea));
					}
					// at this point we have the original clip minus
					// all the overlapping lightweight popups that are painted
					// after this menu item
					g2d.setClip(clip);
				}
			}
		}

		ButtonModel model = menuItem.getModel();
		SubstanceMenu menuUi = (SubstanceMenu) menuItem.getUI();

		Font f = menuItem.getFont();
		g2d.setFont(f);

		// get Accelerator text
		KeyStroke accelerator = menuItem.getAccelerator();
		String acceleratorText = "";
		if (accelerator != null) {
			int modifiers = accelerator.getModifiers();
			if (modifiers > 0) {
				acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
				acceleratorText += UIManager
						.getString("MenuItem.acceleratorDelimiter");
			}

			int keyCode = accelerator.getKeyCode();
			if (keyCode != 0) {
				acceleratorText += KeyEvent.getKeyText(keyCode);
			} else {
				acceleratorText += accelerator.getKeyChar();
			}
		}
		Icon icon = menuItem.getIcon();
		// MenuLayoutInfo mli = MenuUtilities.getMenuLayoutInfo(menuItem, menuUi
		// .getAcceleratorFont(), menuUi.getCheckIcon(), menuUi
		// .getArrowIcon(), menuUi.getDefaultTextIconGap());

		MenuLayoutInfo mli = MenuUtilities.getMenuLayoutInfo(menuItem, menuUi
				.getAcceleratorFont(), menuUi.getCheckIcon(), menuUi
				.getArrowIcon(), menuUi.getDefaultTextIconGap());
		MenuLayoutMetrics popupMetrics = MenuUtilities
				.getPopupLayoutMetrics(menuItem);
		Insets i = menuItem.getInsets();
		// FontMetrics fm = menuItem.getFontMetrics(menuItem.getFont());
		// int fontHeight = fm.getHeight();
		if (popupMetrics != null) {
			MenuGutterFillKind gutterFillKind = SubstanceCoreUtilities
					.getMenuGutterFillKind();
			boolean needExtraIconTextGap = (gutterFillKind != null)
					&& (gutterFillKind != MenuGutterFillKind.NONE);
			int gap = popupMetrics.maxGap;
			if (menuItem.getComponentOrientation().isLeftToRight()) {
				int currX = i.left + gap / 2;
				if (checkIcon != null) {
					mli.checkIconRect = new Rectangle(currX, i.top, checkIcon
							.getIconWidth(), checkIcon.getIconHeight());
					int bump = (popupMetrics.maxCheckIconWidth - checkIcon
							.getIconWidth()) / 2;
					mli.checkIconRect.x += bump;
				}
				if (popupMetrics.maxCheckIconWidth > 0) {
					currX += (popupMetrics.maxCheckIconWidth + gap);
				}
				if (icon != null) {
					mli.iconRect = new Rectangle(currX, i.top, icon
							.getIconWidth(), icon.getIconHeight());
					int bump = (popupMetrics.maxIconWidth - icon.getIconWidth()) / 2;
					mli.iconRect.x += bump;
				}
				if (popupMetrics.maxIconWidth > 0) {
					currX += (popupMetrics.maxIconWidth + gap);
				}
				menuItem.putClientProperty(GUTTER_X, currX + gap / 2);
				if (needExtraIconTextGap)
					currX += gap;
				if (menuItem.getText() != null) {
					mli.textRect = new Rectangle(currX, mli.textRect.y,
							popupMetrics.maxTextWidth, mli.textRect.height);
					mli.text = menuItem.getText();
				}
				currX += (popupMetrics.maxTextWidth + gap);
				if (popupMetrics.maxAcceleratorWidth > 0) {
					currX += 5 * gap;
					// accelerator text is right-aligned
					mli.acceleratorRect = new Rectangle(currX
							+ popupMetrics.maxAcceleratorWidth
							- mli.acceleratorRect.width, mli.textRect.y,
							mli.acceleratorRect.width, mli.textRect.height);
				}
				if (popupMetrics.maxAcceleratorWidth > 0) {
					currX += (popupMetrics.maxAcceleratorWidth + gap);
				}

				if (arrowIcon != null) {
					mli.arrowIconRect = new Rectangle(currX, i.top,
							popupMetrics.maxArrowIconWidth, arrowIcon
									.getIconHeight());
				}

				Rectangle labelRect = new Rectangle(0, 0, menuItem.getWidth(),
						menuItem.getHeight());
				// if (mli.iconRect != null)
				// labelRect = labelRect.union(mli.iconRect);
				if (mli.textRect != null)
					labelRect = mli.textRect;
				// // Align the accelerator text and the check and arrow icons
				// // vertically with the center of the label rect.
				// if (mli.acceleratorRect != null) {
				// mli.acceleratorRect.y = labelRect.y;
				// //+ (labelRect.height / 2)
				// //- (mli.acceleratorRect.height / 2);
				// }
				if (mli.iconRect != null) {
					mli.iconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.iconRect.height / 2);
				}
				if (mli.arrowIconRect != null) {
					mli.arrowIconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.arrowIconRect.height / 2);
				}
				if (mli.checkIconRect != null) {
					mli.checkIconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.checkIconRect.height / 2);
				}
			} else {
				int currX = menuItem.getWidth() - i.right - gap / 2;
				if (checkIcon != null) {
					mli.checkIconRect = new Rectangle(currX
							- popupMetrics.maxCheckIconWidth, i.top, checkIcon
							.getIconWidth(), checkIcon.getIconHeight());
					int bump = (popupMetrics.maxCheckIconWidth - checkIcon
							.getIconWidth()) / 2;
					mli.checkIconRect.x += bump;
				}
				if (popupMetrics.maxCheckIconWidth > 0) {
					currX -= (popupMetrics.maxCheckIconWidth + gap);
				}
				if (icon != null) {
					// icons are center-aligned
					mli.iconRect = new Rectangle(currX
							- popupMetrics.maxIconWidth, i.top, icon
							.getIconWidth(), icon.getIconHeight());
					int bump = (popupMetrics.maxIconWidth - icon.getIconWidth()) / 2;
					mli.iconRect.x += bump;
				}
				if (popupMetrics.maxIconWidth > 0) {
					currX -= (popupMetrics.maxIconWidth + gap);
				}
				menuItem.putClientProperty(GUTTER_X, currX + gap / 2);
				if (needExtraIconTextGap)
					currX -= gap;
				if (menuItem.getText() != null) {
					// text is right-aligned
					mli.textRect = new Rectangle(currX - mli.textRect.width,
							mli.textRect.y, popupMetrics.maxTextWidth,
							mli.textRect.height);
					mli.text = menuItem.getText();
				}
				currX -= (popupMetrics.maxTextWidth + gap);
				if (popupMetrics.maxAcceleratorWidth > 0) {
					currX -= 5 * gap;
					// accelerator text is left-aligned
					mli.acceleratorRect = new Rectangle(currX
							- popupMetrics.maxAcceleratorWidth, mli.textRect.y,
							mli.acceleratorRect.width, mli.textRect.height);
				}
				if (popupMetrics.maxAcceleratorWidth > 0) {
					currX -= (popupMetrics.maxAcceleratorWidth + gap);
				}

				if (arrowIcon != null) {
					mli.arrowIconRect = new Rectangle(currX
							- popupMetrics.maxArrowIconWidth, i.top,
							popupMetrics.maxArrowIconWidth, arrowIcon
									.getIconHeight());
				}

				Rectangle labelRect = new Rectangle(0, 0, menuItem.getWidth(),
						menuItem.getHeight());
				// if (mli.iconRect != null)
				// labelRect = labelRect.union(mli.iconRect);
				if (mli.textRect != null)
					labelRect = mli.textRect;
				// // Align the accelerator text and the check and arrow icons
				// // vertically with the center of the label rect.
				// if (mli.acceleratorRect != null) {
				// mli.acceleratorRect.y = labelRect.y;
				// //+ (labelRect.height / 2)
				// //- (mli.acceleratorRect.height / 2);
				// }
				if (mli.iconRect != null) {
					mli.iconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.iconRect.height / 2);
				}
				if (mli.arrowIconRect != null) {
					mli.arrowIconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.arrowIconRect.height / 2);
				}
				if (mli.checkIconRect != null) {
					mli.checkIconRect.y = labelRect.y + (labelRect.height / 2)
							- (mli.checkIconRect.height / 2);
				}
			}
		}
		Component parent = menuItem.getParent();
		if (parent instanceof JPopupMenu) {
			((JPopupMenu) parent).putClientProperty(GUTTER_X, menuItem
					.getClientProperty(GUTTER_X));
		}

		// MenuUtilities.lineupTextRectangle(menuItem, mli.textRect, menuUi
		// .getAcceleratorFont(), menuUi.getCheckIcon(), menuUi
		// .getArrowIcon(), menuUi.getDefaultTextIconGap());

		SubstanceTextPainter textPainter = SubstanceLookAndFeel
				.getCurrentTextPainter();
		textPainter.init(menuItem, null, true);
		if (textPainter.needsBackgroundImage()) {
			// This is a workaround for disabled menu items. Since
			// SWT (native) pipeline doesn't support painting translucent
			// texts, we need a way to have the background painted in
			// full opacity, but the texts (and highlights) painted with
			// custom translucency. Here, we cheat and first paint the
			// background fill (with watermark) with full opacity. Since
			// there can be no highlights on disabled menu items, we don't
			// need to worry about hightlights. Then, the painting will be done
			// once again, overlaying the background, highlights and
			// text with custom alpha factor.
			paintBackground(g2d, menuItem);
		}
		textPainter.renderSurface(g2d);

		// Paint background
		textPainter.init(menuItem, null, true);
		paintBackground(g2d, menuItem);
		paintHighlights(g2d, menuItem);

		float textAlpha = 1.0f;
		if (mli.text != null) {
			View v = (View) menuItem.getClientProperty(BasicHTML.propertyKey);
			if (v != null) {
				v.paint(g2d, mli.textRect);
			} else {
				textAlpha = SubstanceCoreUtilities.paintText(menuItem,
						mli.textRect, mli.text, menuItem
								.getDisplayedMnemonicIndex());
			}
		}

		// Draw the Accelerator Text
		if (acceleratorText != null && !acceleratorText.equals("")) {
			SubstanceCoreUtilities.paintText(menuItem, mli.acceleratorRect,
					acceleratorText, -1);
		}

		Graphics2D graphics = (Graphics2D) g2d.create();
		graphics.setComposite(TransitionLayout.getAlphaComposite(menuItem,
				textAlpha, g2d));

		textPainter.renderSurface(graphics);
		graphics.dispose();

		// Paint the Check
		if (checkIcon != null) {
			if (useCheckAndArrow(menuItem))
				checkIcon.paintIcon(menuItem, g2d, mli.checkIconRect.x,
						mli.checkIconRect.y);
		}

		// Paint the Icon
		if (icon != null) {
			if (!model.isEnabled()) {
				icon = menuItem.getDisabledIcon();
			} else if (model.isPressed() && model.isArmed()) {
				icon = menuItem.getPressedIcon();
				if (icon == null) {
					// Use default icon
					icon = menuItem.getIcon();
				}
			} else {
				icon = menuItem.getIcon();
			}

			if (icon != null) {
				icon.paintIcon(menuItem, g2d, mli.iconRect.x, mli.iconRect.y);
			}
		}

		// Paint the Arrow
		if (arrowIcon != null) {
			if (useCheckAndArrow(menuItem))
				arrowIcon.paintIcon(menuItem, g2d, mli.arrowIconRect.x,
						mli.arrowIconRect.y);
		}

		// g2d.setColor(Color.red);
		// if (mli.iconRect != null)
		// g2d.draw(mli.iconRect);
		// if (mli.checkIconRect != null)
		// g2d.draw(mli.checkIconRect);
		// if (mli.textRect != null)
		// g2d.draw(mli.textRect);

		// g2d.setColor(Color.red);
		// g2d.drawRect(mli.checkIconRect.x, mli.checkIconRect.y,
		// mli.checkIconRect.width, mli.checkIconRect.height);
		// g2d.drawRect(mli.iconRect.x, mli.iconRect.y, mli.iconRect.width,
		// mli.iconRect.height);
		// g2d.drawRect(mli.textRect.x, mli.textRect.y, mli.textRect.width,
		// mli.textRect.height);
		// g2d.drawRect(mli.arrowIconRect.x, mli.arrowIconRect.y,
		// mli.arrowIconRect.width, mli.arrowIconRect.height);
		// g2d.drawRect(mli.acceleratorRect.x, mli.acceleratorRect.y,
		// mli.acceleratorRect.width, mli.acceleratorRect.height);

		g2d.dispose();
	}

	protected static void paintBackground(Graphics g, final JMenuItem menuItem) {
		int textOffset = MenuUtilities.getTextOffset(menuItem, menuItem
				.getParent());
		// System.out.println(menuItem.getText() + ":" + textOffset);
		SubstanceTextPainter textPainter = SubstanceLookAndFeel
				.getCurrentTextPainter();
		final Color bgColor = SubstanceCoreUtilities
				.getBackgroundFillColor(menuItem);
		final int finalTextOffset = textOffset;
		SubstanceTextPainter.BackgroundPaintingCallback callback = new SubstanceTextPainter.BackgroundPaintingCallback() {
			public void paintBackground(Graphics g) {
				Graphics2D g2d = (Graphics2D) g.create();
				backgroundDelegate.paintBackground(g, menuItem, bgColor, 0.5f,
						finalTextOffset);
				g2d.dispose();
			}
		};
		if (textPainter.needsBackgroundImage()) {
			textPainter.setBackgroundFill(menuItem, bgColor, true, 0, 0);
			textPainter.attachCallback(callback);
		} else {
			callback.paintBackground(g);
		}
	}

	protected static void paintHighlights(Graphics g, final JMenuItem menuItem) {
		SubstanceTextPainter textPainter = SubstanceLookAndFeel
				.getCurrentTextPainter();
		SubstanceTextPainter.BackgroundPaintingCallback callback = new SubstanceTextPainter.BackgroundPaintingCallback() {
			public void paintBackground(Graphics g) {
				Graphics2D g2d = (Graphics2D) g.create();
				SubstanceMenuBackgroundDelegate.paintHighlights(g, menuItem,
						0.5f);
				g2d.dispose();
			}
		};
		if (textPainter.needsBackgroundImage()) {
			textPainter.attachCallback(callback);
		} else {
			callback.paintBackground(g);
		}
	}

	protected static MenuLayoutMetrics getMetrics(JPopupMenu popupMenu) {
		MenuLayoutMetrics metrics = new MenuLayoutMetrics();
		for (int i = 0; i < popupMenu.getComponentCount(); i++) {
			Component comp = popupMenu.getComponent(i);
			if (comp instanceof JMenuItem) {
				JMenuItem childMenuItem = (JMenuItem) comp;
				ButtonUI bui = childMenuItem.getUI();
				if (!(bui instanceof SubstanceMenu))
					continue;
				SubstanceMenu ui = (SubstanceMenu) bui;
				MenuLayoutInfo mli = MenuUtilities.getMenuLayoutInfo(
						childMenuItem, ui.getAcceleratorFont(), ui
								.getCheckIcon(), ui.getArrowIcon(), ui
								.getDefaultTextIconGap());
				metrics.maxIconWidth = Math.max(metrics.maxIconWidth,
						mli.iconRect.width);
				metrics.maxCheckIconWidth = Math.max(metrics.maxCheckIconWidth,
						mli.checkIconRect.width);
				metrics.maxTextWidth = Math.max(metrics.maxTextWidth,
						mli.textRect.width);
				metrics.maxAcceleratorWidth = Math.max(
						metrics.maxAcceleratorWidth, mli.acceleratorRect.width);
				metrics.maxArrowIconWidth = Math.max(metrics.maxArrowIconWidth,
						mli.arrowIconRect.width);
				metrics.maxGap = Math.max(metrics.maxGap, ui
						.getDefaultTextIconGap());
			}
		}
		return metrics;
	}

	public static MenuLayoutMetrics getPopupLayoutMetrics(JMenuItem menuItem) {
		Component comp = menuItem.getParent();
		if (comp instanceof JPopupMenu) {
			JPopupMenu popupMenu = (JPopupMenu) comp;
			Object prop = popupMenu.getClientProperty(LAYOUT_METRICS);
			if (prop instanceof MenuLayoutMetrics) {
				return (MenuLayoutMetrics) prop;
			}
			MenuLayoutMetrics metrics = getMetrics(popupMenu);
			popupMenu.putClientProperty(LAYOUT_METRICS, metrics);
			return metrics;
		}

		// fix for issue 347 - menu item in menu bar is not
		// covered by the code above
		if (!(comp instanceof JMenu)) {
			ButtonUI bui = menuItem.getUI();
			if (bui instanceof SubstanceMenu) {
				SubstanceMenu ui = (SubstanceMenu) bui;

				MenuLayoutInfo mli = MenuUtilities.getMenuLayoutInfo(menuItem,
						ui.getAcceleratorFont(), ui.getCheckIcon(), ui
								.getArrowIcon(), ui.getDefaultTextIconGap());
				MenuLayoutMetrics metrics = new MenuLayoutMetrics();
				metrics.maxIconWidth = mli.iconRect.width;
				metrics.maxCheckIconWidth = mli.checkIconRect.width;
				metrics.maxTextWidth = mli.textRect.width;
				metrics.maxAcceleratorWidth = mli.acceleratorRect.width;
				metrics.maxArrowIconWidth = mli.arrowIconRect.width;
				metrics.maxGap = ui.getDefaultTextIconGap();
				return metrics;
			}
		}

		// If here, the parent is not popup menu - this is a top-level menu
		// in a menu bar. No need to align the different rectangles
		return null;
	}

	public static void cleanPopupLayoutMetrics(JMenuItem menuItem) {
		Component comp = menuItem.getParent();
		if (comp instanceof JPopupMenu) {
			JPopupMenu popupMenu = (JPopupMenu) comp;
			cleanPopupLayoutMetrics(popupMenu);
		}
	}

	public static void cleanPopupLayoutMetrics(JPopupMenu popupMenu) {
		if (popupMenu != null) {
			popupMenu.putClientProperty(LAYOUT_METRICS, null);
		}
	}

	public static int getPreferredWidth(JMenuItem menuItem) {
		Insets ins = menuItem.getInsets();
		MenuLayoutMetrics popupMetrics = MenuUtilities
				.getPopupLayoutMetrics(menuItem);

		int width = popupMetrics.maxCheckIconWidth + popupMetrics.maxIconWidth
				+ popupMetrics.maxTextWidth + popupMetrics.maxAcceleratorWidth
				+ popupMetrics.maxArrowIconWidth + ins.left + ins.right;

		int gapsToAdd = 0;
		if (popupMetrics.maxCheckIconWidth > 0)
			gapsToAdd++;
		if (popupMetrics.maxIconWidth > 0)
			gapsToAdd++;
		if (popupMetrics.maxAcceleratorWidth > 0)
			gapsToAdd++;
		if (popupMetrics.maxArrowIconWidth > 0)
			gapsToAdd++;

		int gap = popupMetrics.maxGap;
		width += ((1 + gapsToAdd) * gap);

		if (popupMetrics.maxAcceleratorWidth > 0) {
			// at least one menu item has accelerator text
			width += 5 * gap;
		}

		MenuGutterFillKind gutterFillKind = SubstanceCoreUtilities
				.getMenuGutterFillKind();
		boolean needExtraIconTextGap = (gutterFillKind != null)
				&& (gutterFillKind != MenuGutterFillKind.NONE);
		if (needExtraIconTextGap)
			width += gap;

		return width;
	}

	public static int getTextOffset(JComponent menuItem,
			Component menuItemParent) {
		if (!(menuItemParent instanceof JPopupMenu)) {
			return 0;
		}

		Object itemProp = menuItem.getClientProperty(GUTTER_X);
		if (itemProp instanceof Integer) {
			return (Integer) itemProp;
		}

		JPopupMenu popupMenu = (JPopupMenu) menuItemParent;
		Object parentProp = popupMenu.getClientProperty(GUTTER_X);
		if (parentProp instanceof Integer) {
			return (Integer) parentProp;
		}
		return 0;
		// MenuLayoutMetrics metrics = getMetrics(popupMenu);
		// if (popupMenu.getComponentOrientation().isLeftToRight()) {
		// return SubstanceSizeUtils.getMenuItemMargin(SubstanceSizeUtils
		// .getComponentFontSize(menuItem))
		// + popupMenu.getInsets().left
		// + metrics.maxCheckIconWidth
		// + metrics.maxIconWidth
		// + (int) (2.5 * SubstanceSizeUtils
		// .getMenuTextIconGap(SubstanceSizeUtils
		// .getComponentFontSize(popupMenu)));
		// } else {
		// return 0;
		// }
	}
}
